.mark {
    background-color: yellow;
    color: red;
    display:inline-block;
}

.burk {
    background-color: red;
    color: yellow;
    display:inline-block;
}

.girk {
    background-color: lime;
    color: red;
    display:inline-block;
}
